#!/usr/bin/env python
#
# Copyright (c) 2019, Hewlett Packard Enterprise Development LP
#

from __future__ import print_function
from __future__ import with_statement

import re
import sys, cmd
from role import Role
from logo import Logo
from image import Image
from attach import Attach
from define import Define
from baseimg import Baseimg
from builder import Builder
from catalog import Catalog
from service import Service
from sources import Sources
from appconfig import Appconfig
from workbench import Workbench
from clusterconfig import ClusterConfiguration
from document import Document

from utils import isDebug
from utils.config import WBConfig
from utils.misc import calculateAndValidateDockerVersion
from constants import AWB_VERSION
from inmem_store import InmemStore
from inmem_store import VARIABLES_DICT

__all__ = ['BDwb']

def appendLine(retline, newline):
    if retline == '':
        return newline
    else:
        return retline + " " + newline

class Batchfile(object):
    def __init__(self, filepath):
        with open(filepath, 'r') as f:
            self.lines = f.readlines()

    def close(self):
        return

    def readline(self):
        haveLine = False
        retLine = ''

        while not haveLine:
            if len(self.lines) == 0:
                return retLine
            else:
                line = self.lines.pop(0).rstrip('\r\n').strip()

            if (line == '') or line.startswith('#'):
                continue

            if line.endswith('\\'):
                line = line.rstrip('\\').strip()
                retLine = appendLine(retLine, line)
                haveLine = False
                continue
            else:
                retLine = appendLine(retLine, line)
                haveLine = True

        return retLine

class BDwb(cmd.Cmd):

    def __init__(self, instr=None, batchfile=None):
        self.config = WBConfig()
        self.batchfile = batchfile
        self.inmemStore = InmemStore()

        self.commands = {
                'role'    : Role(self, self.config, self.inmemStore),
                'logo'    : Logo(self, self.config, self.inmemStore),
                'image'   : Image(self, self.config, self.inmemStore),
                'attach'  : Attach(self, self.config, self.inmemStore),
                'define'  : Define(self, self.config, self.inmemStore),
                'baseimg' : Baseimg(self, self.config, self.inmemStore),
                'builder' : Builder(self, self.config, self.inmemStore),
                'catalog' : Catalog(self, self.config, self.inmemStore),
                'service' : Service(self, self.config, self.inmemStore),
                'sources' : Sources(self, self.config, self.inmemStore),
                'appconfig' : Appconfig(self, self.config, self.inmemStore),
                'workbench' : Workbench(self, self.config, self.inmemStore),
                'clusterconfig' : ClusterConfiguration(self, self.config, self.inmemStore),
                'document'  : Document(self, self.config, self.inmemStore)
        }

        self.ruler = '_'
        defIntro = "BlueData workbench version %s.\n" %(AWB_VERSION)
        if self.batchfile:
            self.intro = defIntro + "Executing in non-interative mode."
            self.prompt = ''
            self.use_rawinput = False
            cmd.Cmd.__init__(self, stdin=Batchfile(self.batchfile))
        else:
            self.intro = defIntro + "Executing in interactive mode."
            self.prompt = 'bdwb> '
            self.use_rawinput = True
            cmd.Cmd.__init__(self)

        if instr != None:
            sys.exit(not self.onecmd(instr))

        if not calculateAndValidateDockerVersion(self.inmemStore):
            sys.exit(1)

    def emptyline(self):
        """
        Override this method so we don't automatically execute the previous cmd.
        """
        # do nothing.
        return

    def precmd(self, line):
        """
        This overridden method added for convienience when the user executes
        help. Normally for a subcommand help, the user must do the following:

            bdwb> catalog new -h

        instead, this override lets them do the following:

            bdwb> help catalog new

        The previous cmd will still work if the user happens to use it.
        """
        splits = line.split()

        if (len(splits) > 1) and splits[0] == 'help':
            if not (splits[1] == 'EOF' or splits[1] == 'exit'):
                return ' '.join(splits[1:] + ['-h'])


        # Perform any variable replacements in the line string.
        variablesDict = self.inmemStore.getDict(VARIABLES_DICT)
        substitutedSplits = []
        for s in splits:
            m = re.match(r'.*%(.*)%.*', s)
            if hasattr(m, 'groups'):
                var = m.groups()[0]
                if variablesDict.has_key(var):
                    newStr = s.replace('%%%s%%' %(var), variablesDict[var])
                    substitutedSplits.append(newStr)
                else:
                    print("WARNING: Variable '%s' referenced before definition" % (var))

                    # Leave the variable in the command line. This may help
                    # if the developer really wanted to use a value that begins
                    # and ends with a percentage (%) sign for what ever reason.
                    substitutedSplits.append(s)
            else:
                substitutedSplits.append(s)

        finalCmdline = ' '.join(substitutedSplits)
        if isDebug():
            print("EXECUTING:", finalCmdline)

        return finalCmdline

    def postcmd(self, cont, line):
        """
        This overridden method will stop the command loop depending on the return
        status of the command executed. This check only applies when executing
        in batch mode.
        """
        if cont == False:
            print("\n", "ERROR: Instruction failed: %s" % (line))
            if self.use_rawinput == False:
                return True ## stop?

        return False ## stop?

    # ##############################################################
    # #         Use '!' to execute normal shell cmds               #
    # ##############################################################
    # def do_shell(self, line):
    #     output = os.popen(line).read()
    #     print(output)
    #     self.last_output = output

    ##############################################################
    #         workbench command                                  #
    ##############################################################
    def do_workbench(self, line):
        return self.commands['workbench'].run(line)

    def help_workbench(self):
        self.commands['workbench'].help()

    ##############################################################
    #         builder command                                    #
    ##############################################################
    def do_builder(self, line):
        return self.commands['builder'].run(line)

    def help_builder(self):
        self.commands['builder'].help()

    ##############################################################
    #         catalog command                                    #
    ##############################################################
    def do_catalog(self, line):
        return self.commands['catalog'].run(line)

    def help_catalog(self):
        self.commands['catalog'].help()

    ##############################################################
    #         role command                                       #
    ##############################################################
    def do_role(self, line):
        return self.commands['role'].run(line)

    def help_role(self):
        self.commands['role'].help()

    ##############################################################
    #         logo command                                       #
    ##############################################################
    def do_logo(self, line):
        return self.commands['logo'].run(line)

    def help_logo(self):
        self.commands['logo'].help()

    ##############################################################
    #         attach command                                     #
    ##############################################################
    def do_attach(self, line):
        return self.commands['attach'].run(line)

    def help_attach(self):
        self.commands['attach'].help()

    ##############################################################
    #         image command                                      #
    ##############################################################
    def do_image(self, line):
        return self.commands['image'].run(line)

    def help_image(self):
        self.commands['image'].help()

    ##############################################################
    #         define command                                     #
    ##############################################################
    def do_define(self, line):
        return self.commands['define'].run(line)

    def help_define(self):
        self.commands['define'].help()

    ##############################################################
    #         Base image command                                 #
    ##############################################################
    def do_baseimg(self, line):
        return self.commands['baseimg'].run(line)

    def help_baseimg(self):
        self.commands['baseimg'].help()

    ##############################################################
    #         appconfig command                                  #
    ##############################################################
    def do_appconfig(self, line):
        return self.commands['appconfig'].run(line)

    def help_appconfig(self):
        self.commands['appconfig'].help()

    ##############################################################
    #         service command                                    #
    ##############################################################
    def do_service(self, line):
        return self.commands['service'].run(line)

    def help_service(self):
        self.commands['service'].help()

    ##############################################################
    #         sources commands                                   #
    ##############################################################
    def do_sources(self, line):
        return self.commands['sources'].run(line)

    def help_sources(self):
        self.commands['sources'].help()

    ##############################################################
    #         configuration commands                             #
    ##############################################################
    def do_clusterconfig(self, line):
        return self.commands['clusterconfig'].run(line)

    def help_clusterconfig(self):
        self.commands['clusterconfig'].help()

    ##############################################################
    #         document command                                   #
    ##############################################################
    def do_document(self, line):
        return self.commands['document'].run(line)

    def help_document(self):
        self.commands['document'].help()

    ##############################################################
    #       default complete function                            #
    ##############################################################
    def completedefault(self, *ignored):
        (text, line, begidx, endidx) = ignored
        command = line.strip().split()[0]
        if self.commands.has_key(command):
            return self.commands[command].complete(text, line, begidx, endidx)
        else:
            return cmd.Cmd.completedefault(self, ignored)


    ##############################################################
    #                Exit the interactive shell                  #
    ##############################################################
    def do_exit(self, line):
        print("\n")
        sys.exit(0)

    def do_EOF(self, line):
        return self.do_exit(line)

    def help_exit(self):
        print('\n'.join(["exit | Ctrl+D", "\tExits the interactive shell."]))

    def help_EOF(self):
        self.help_exit()
