#!/bin/env python
#
# Copyright (c) 2019, Hewlett Packard Enterprise Development LP

from __future__ import print_function
from .. import Command
from ..constants import *
from ..inmem_store import ENTRY_DICT, DELIVERABLE_DICT
from ..utils import executeShellCmd
from ..utils.misc import constructDistroId

from .image_load import ImageLoad
from .image_list import ImageList
from .image_pull import ImagePull
from .image_push import ImagePush
from .image_build import ImageBuild
from .image_package import ImagePackage
from .image_download import ImageDownload
from .image_registry import ImageRegistry

import os

DOCKER_LOGIN_CMD_FMT = "docker login -u '%(username)s' -p '%(password)s' %(email)s %(registryurl)s"

class Image(Command):
    """

    """

    class DockerLoginContext(object):
        """

        """

        def __init__(self, builtOnDocker, registryUrl, pargs):
            self.registryUrl = '' if registryUrl in ['docker.io', 'default'] else registryUrl
            self.emailString = "-e 'default@default.com'" if (builtOnDocker == "1.7") else ''

            self.username = pargs.username if pargs.username else              \
                                os.getenv(AWB_REGISTRY_USERNAME, None)
            self.password = pargs.password if pargs.password else              \
                                os.getenv(AWB_REGISTRY_PASSWORD, None)
            self.authEnabled = (self.username and self.password)

            self.ctRoot = pargs.ctroot if pargs.ctroot else                         \
                                os.getenv(DOCKER_CONTENT_TRUST_ROOT_PASSPHRASE, None)
            self.ctRegistry = pargs.ctregistry if pargs.ctregistry  else            \
                                os.getenv(DOCKER_CONTENT_TRUST_REPOSITORY_PASSPHRASE, None)
            self.contentTrustEnabled = (self.ctRoot and self.ctRegistry)

        def __enter__(self):
            if (not self.authEnabled) or (self.registryUrl == None):
                return self

            loginCmd = DOCKER_LOGIN_CMD_FMT % {'username'    : self.username,
                                               'password'    : self.password,
                                               'email'       : self.emailString,
                                               'registryurl' : self.registryUrl}
            altLoginCmd = DOCKER_LOGIN_CMD_FMT % {'username'    : self.username,
                                                  'password'    : "<<hidden>>",
                                                  'email'       : self.emailString,
                                                  'registryurl' : self.registryUrl}
            # Login
            if not executeShellCmd(loginCmd, alternateStr=altLoginCmd):
                print ("ERROR: Failed to login to registry")
                return False

            return self

        def __exit__(self, _type, _value, _traceback):
            if (not self.authEnabled) or (self.registryUrl == None):
                return

            logoutCmd = "docker logout %s" % (self.registryUrl)
            executeShellCmd(logoutCmd)

    def __init__(self, wb, config, inmemStore):
        Command.__init__(self, wb, config, inmemStore, 'image',
                         'Container image management for the catalog entry.')

        ## Initialize the subcommands.
        ImageLoad(config, inmemStore, self)
        ImageList(config, inmemStore, self)
        ImagePull(config, inmemStore, self)
        ImagePush(config, inmemStore, self)
        ImageBuild(config, inmemStore, self)
        ImagePackage(config, inmemStore, self)
        ImageDownload(config, inmemStore, self)
        ImageRegistry(config, inmemStore, self)

    def getDockerLoginContext(self, builtOnDocker, registryUrl, pargs):
        return Image.DockerLoginContext(builtOnDocker, registryUrl, pargs)

    def addAuthAndCTArgs(self, subparser):
        """
        """
        authGroup = subparser.add_argument_group('For authentication-enabled registry.')
        authGroup.add_argument('-u', '--username', metavar='USERNAME', type=str,
                                dest='username', default=None, required=False,
                                help='Specify the username for pushing the image '
                                'to an authentication-enabled registry. You may'
                                'also set the environment variable AWB_REGISTRY_USERNAME.')
        authGroup.add_argument('-p', '--password', metavar='PASSWORD', type=str,
                                dest='password', default=None, required=False,
                                help='Specify the password for pushing to an '
                                'authentication-enabled registry. You may also set '
                                'the environment variable AWB_REGISTRY_PASSWORD.')

        ctGroup = subparser.add_argument_group('For content trust enabled registry.')
        ctGroup.add_argument('-o', '--ct-root-passphrase', metavar='PASSPHRASE',
                             type=str, dest='ctroot', required=False, default=None,
                             help='Specify the content trust root passphrase if'
                             'content trust is enabled for the repository. You may'
                             'also set the environment variable DOCKER_CONTENT_TRUST_ROOT_PASSPHRASE')
        ctGroup.add_argument('-r', '--ct-registry-passphrase', metavar='PASSPHRASE',
                             type=str, dest='ctregistry', required=False, default=None,
                             help='Specify the content trust repository passphrase if'
                             'content trust is enabled for the repository. You may'
                             'also set the environment variable DOCKER_CONTENT_TRUST_REPOSITORY_PASSPHRASE')

    def stringify_roles(self, roles):
        """
        """
        return "--roles %s" %(' '.join(roles)) if roles != DEFAULT_IMAGE_ROLES else ''


    def parseImageRepotag(self, inputRepoTag):
        """
        Parse the given image repotag and return the registry part of it
        seperately if specified.
        """
        usableInputRepoTag = inputRepoTag.strip()\
                                         .replace('http://', '')\
                                         .replace('https://', '')\
                                         .lower()
        repotagSplits = usableInputRepoTag.split('/')
        if len(repotagSplits) > 3:
            print("ERROR: Bad image repotag specified: ", inputRepoTag)
            registryUrl = None
            repotag = None
        elif len(repotagSplits) == 3:
            registryUrl = repotagSplits[0]
            repotag = '/'.join([repotagSplits[1], repotagSplits[2]])
        elif len(repotagSplits) == 2:
            registryUrl = None
            repotag = '/'.join([repotagSplits[0], repotagSplits[1]])
        else:
            registryUrl = None
            repotag = repotagSplits[0]

        return (registryUrl, repotag)

Command.register(Image)
__all__ = ['Image', 'DockerLoginContext']
